﻿using System;
using System.Collections.Generic;
using System.IO;
using UnityEngine;

public class DataPointProvider : MonoBehaviour
{
  [Serializable]
  public struct PointListContainer
  {
    public DataPath.DataPoint[] dataPoints;
  }

  private PointListContainer pointListContainer;
  public DataPath.DataPoint[] pointList
  {
    get
    {
      return pointListContainer.dataPoints;
    }
  }
  public TextAsset data;

  public void Awake()
  {
    int hash = this.data.text.GetHashCode();

    string dataBlobPath = UnityEngine.Windows.Directory.temporaryFolder + "/" + hash;

    var stopWatch = new System.Diagnostics.Stopwatch();
    stopWatch.Start();
    bool readFromBlob = File.Exists(dataBlobPath);

    if (readFromBlob)
    {
      try
      {
        ReadDataBlob(dataBlobPath);
      }
      catch (Exception ex)
      {
        Debug.LogError(ex.Message);
        readFromBlob = false;
      }
    }
    
    if(!readFromBlob)
    {
      ReadCSVFile();
      WriteDataBlob(dataBlobPath);
    }

    stopWatch.Stop();
    Debug.Log(stopWatch.Elapsed.TotalSeconds);
  }

  private void ReadDataBlob(string dataBlobPath)
  {
    var formatter = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
    using (Stream stream = new FileStream(dataBlobPath, FileMode.Open, FileAccess.Read))
    {
      pointListContainer = (PointListContainer)formatter.Deserialize(stream);
    }
  }

  private void WriteDataBlob(string dataBlobPath)
  {
    Debug.Log("Write To " + dataBlobPath);
    var formatter = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
    using (Stream stream = new FileStream(dataBlobPath, FileMode.Create, FileAccess.Write))
    {
      formatter.Serialize(stream, this.pointListContainer);
      stream.Close();
    }
  }

  private void ReadCSVFile()
  {
    using (var rdr = new StringReader(this.data.text))
    {
      this.pointListContainer = new PointListContainer();
      List<DataPath.DataPointCSV> pointList = Sinbad.CsvUtil.LoadObjects<DataPath.DataPointCSV>(rdr);

      //objs.Sort(Comparer<DataPath.DataPoint>.Create((l,r) => l.direction - r.direction));
      //objs.Sort(Comparer<DataPath.DataPoint>.Create((l,r) => isign(l.timestamp - r.timestamp)));
      //objs.Sort(Comparer<DataPath.DataPoint>.Create((l,r) => l.id.GetHashCode() - r.id.GetHashCode()));
      pointList.Sort(Comparer<DataPath.DataPointCSV>.Create((l, r) => l.id.CompareTo(r.id)));

      this.pointListContainer.dataPoints = new DataPath.DataPoint[pointList.Count];
      for (int i = 0; i != pointList.Count; ++i)
      {
        var dp = pointList[i].ToDataPoint();
        this.pointListContainer.dataPoints[i] = dp;
      }
    }
  }
}